diff --git a/DataStr/List.cpp b/DataStr/List.cpp
index 93142b6f39f93cc7e5fc8c976a80744d0dc6775f..3517fcab72d5f11df1cbc3b5f0868e0991a76a6c 100644
--- a/DataStr/List.cpp
+++ b/DataStr/List.cpp
@@ -1,4 +1,4 @@
-// $Id: List.cpp,v 1.40 2007-09-04 13:47:00 remacle Exp $
+// $Id: List.cpp,v 1.41 2007-09-08 21:26:03 geuzaine Exp $
 //
 // Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
 //
@@ -407,6 +407,16 @@ void List_Copy(List_T * a, List_T * b)
   }
 }
 
+void List_Merge(List_T * a, List_T * b)
+{
+  int i;
+
+  if(!a || !b) return;
+  for(i = 0; i < List_Nbr(a); i++) {
+    List_Add(b, List_Pointer_Fast(a, i));
+  }
+}
+
 void swap_bytes(char *array, int size, int n)
 {
   int i, c;
diff --git a/DataStr/List.h b/DataStr/List.h
index c47d2ae0da5829e0c0bf592930c76790e982a563..50d146800a6953bbd1444658f5dd2e49b7bd654b 100644
--- a/DataStr/List.h
+++ b/DataStr/List.h
@@ -69,6 +69,7 @@ void    List_Reset(List_T *liste);
 void    List_Action(List_T *liste, void (*action)(void *data, void *dummy));
 void    List_Action_Inverse(List_T *liste, void (*action)(void *data, void *dummy));
 void    List_Copy(List_T *a , List_T *b);
+void    List_Merge(List_T *a , List_T *b);
 List_T *List_CreateFromFile(int n, int incr, int size, FILE *file, int format, int swap);
 void    List_WriteToFile(List_T *liste, FILE *file, int format);
 
diff --git a/Fltk/Makefile b/Fltk/Makefile
index 7088f61b191585ae2f09330d1d886cf162ddb16f..f24e49fdd80f870b4dce01ef35a7350c59a30c7e 100644
--- a/Fltk/Makefile
+++ b/Fltk/Makefile
@@ -1,4 +1,4 @@
-# $Id: Makefile,v 1.146 2007-09-04 15:19:24 geuzaine Exp $
+# $Id: Makefile,v 1.147 2007-09-08 21:26:04 geuzaine Exp $
 #
 # Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
 #
@@ -91,8 +91,8 @@ Main.o: Main.cpp GUI.h Opengl_Window.h Colorbar_Window.h \
   ../Geo/GRegion.h ../Geo/GEntity.h ../Geo/MElement.h \
   ../Geo/ExtrudeParams.h ../Geo/SBoundingBox3d.h ../Mesh/Field.h \
   ../Post/OctreePost.h ../Common/Octree.h ../Common/OctreeInternals.h \
-  ../Mesh/BackgroundMesh.h ../Post/PView.h ../Post/AdaptiveViews.h \
-  ../Post/PViewData.h ../Post/PViewOptions.h ../Post/ColorTable.h
+  ../Mesh/BackgroundMesh.h ../Post/PView.h ../Post/PViewData.h \
+  ../Post/PViewOptions.h ../Post/ColorTable.h
 Message.o: Message.cpp ../Common/Gmsh.h ../Common/Message.h \
   ../DataStr/Malloc.h ../DataStr/List.h ../DataStr/Tree.h \
   ../DataStr/avl.h ../DataStr/Tools.h ../DataStr/List.h ../DataStr/Tree.h \
diff --git a/Geo/Geo.cpp b/Geo/Geo.cpp
index 66736de4a1ce1b68f4fdb6bb4d283d34da398b7c..f1bddc384d69371c46c429b7c65564e5fcee0894 100644
--- a/Geo/Geo.cpp
+++ b/Geo/Geo.cpp
@@ -1,4 +1,4 @@
-// $Id: Geo.cpp,v 1.95 2007-09-05 10:11:30 geuzaine Exp $
+// $Id: Geo.cpp,v 1.96 2007-09-08 21:26:04 geuzaine Exp $
 //
 // Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
 //
@@ -1794,18 +1794,20 @@ void BoundaryShapes(List_T *shapes, List_T *shapesBoundary)
     List_Read(shapes, i, &O);
     switch (O.Type) {
     case MSH_POINT:
+    case MSH_POINT_BND_LAYER:
       return;
       break;
     case MSH_SEGM_LINE:
     case MSH_SEGM_SPLN:
-    case MSH_SEGM_BSPLN:
-    case MSH_SEGM_BEZIER:
     case MSH_SEGM_CIRC:
     case MSH_SEGM_CIRC_INV:
     case MSH_SEGM_ELLI:
     case MSH_SEGM_ELLI_INV:
+    case MSH_SEGM_BSPLN:
     case MSH_SEGM_NURBS:
+    case MSH_SEGM_BEZIER:
     case MSH_SEGM_PARAMETRIC:
+    case MSH_SEGM_BND_LAYER:
       {
 	Curve *c = FindCurve(O.Num);
 	if(c){
@@ -1826,9 +1828,10 @@ void BoundaryShapes(List_T *shapes, List_T *shapesBoundary)
 	  Msg(GERROR, "Unknown curve %d", O.Num);
       }
       break;
+    case MSH_SURF_PLAN:
     case MSH_SURF_REGL:
     case MSH_SURF_TRIC:
-    case MSH_SURF_PLAN:
+    case MSH_SURF_BND_LAYER:
       {
 	Surface *s = FindSurface(O.Num);
 	if(s){
diff --git a/Graphics/Graph2D_Old.cpp b/Graphics/Graph2D_Old.cpp
index 4ef54d9a5390611c4dfce372c04dd47fd004fb95..5939a6670906b4d169941f581d586a2b65266ded 100644
--- a/Graphics/Graph2D_Old.cpp
+++ b/Graphics/Graph2D_Old.cpp
@@ -1,4 +1,4 @@
-// $Id: Graph2D_Old.cpp,v 1.1 2007-08-29 18:41:06 geuzaine Exp $
+// $Id: Graph2D_Old.cpp,v 1.2 2007-09-08 21:26:04 geuzaine Exp $
 //
 // Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
 //
@@ -526,8 +526,6 @@ void Draw_Text2D3D(int dim, int timestep, int nb, List_T * td, List_T * tc)
   }
 }
 
-#include "PView.h"
-
 void Draw_Text2D_Old()
 {
   if(!CTX.post.list)
diff --git a/Graphics/Makefile b/Graphics/Makefile
index 5b65b8f410a6c07fc62a5a41a9370392830f623f..c983d970af4f6406ebddd805d8d1bd2f64cd0b54 100644
--- a/Graphics/Makefile
+++ b/Graphics/Makefile
@@ -1,4 +1,4 @@
-# $Id: Makefile,v 1.126 2007-09-01 16:05:43 geuzaine Exp $
+# $Id: Makefile,v 1.127 2007-09-08 21:26:04 geuzaine Exp $
 #
 # Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
 #
@@ -135,8 +135,8 @@ Post.o: Post.cpp ../Common/Gmsh.h ../Common/Message.h ../DataStr/Malloc.h \
   Iso.h ../Post/Views.h ../Post/ColorTable.h ../Common/VertexArray.h \
   ../Geo/SVector3.h ../Geo/SPoint3.h ../Common/SmoothData.h \
   ../Post/AdaptiveViews.h ../Common/GmshMatrix.h ../Post/PView.h \
-  ../Post/AdaptiveViews.h ../Post/PViewData.h ../Post/PViewOptions.h \
-  ../Post/ColorTable.h ../Common/Context.h gl2ps.h
+  ../Post/PViewData.h ../Post/PViewOptions.h ../Post/ColorTable.h \
+  ../Common/Context.h gl2ps.h
 PostElement.o: PostElement.cpp ../Common/Gmsh.h ../Common/Message.h \
   ../DataStr/Malloc.h ../DataStr/List.h ../DataStr/Tree.h \
   ../DataStr/avl.h ../DataStr/Tools.h ../DataStr/List.h ../DataStr/Tree.h \
@@ -199,9 +199,8 @@ Scale_Old.o: Scale_Old.cpp ../Common/Gmsh.h ../Common/Message.h \
 Scale.o: Scale.cpp ../Common/GmshUI.h Draw.h ../DataStr/List.h \
   ../Geo/SBoundingBox3d.h ../Geo/SPoint3.h ../Post/PView.h \
   ../Common/VertexArray.h ../Geo/SVector3.h ../Geo/SPoint3.h \
-  ../Common/SmoothData.h ../Numeric/Numeric.h ../Post/AdaptiveViews.h \
-  ../Common/GmshMatrix.h ../Post/PViewData.h ../Post/PViewOptions.h \
-  ../Post/ColorTable.h ../Common/Context.h gl2ps.h
+  ../Common/SmoothData.h ../Numeric/Numeric.h ../Post/PViewData.h \
+  ../Post/PViewOptions.h ../Post/ColorTable.h ../Common/Context.h gl2ps.h
 Graph2D_Old.o: Graph2D_Old.cpp ../Common/Gmsh.h ../Common/Message.h \
   ../DataStr/Malloc.h ../DataStr/List.h ../DataStr/Tree.h \
   ../DataStr/avl.h ../DataStr/Tools.h ../DataStr/List.h ../DataStr/Tree.h \
@@ -209,14 +208,12 @@ Graph2D_Old.o: Graph2D_Old.cpp ../Common/Gmsh.h ../Common/Message.h \
   ../Geo/SBoundingBox3d.h ../Geo/SPoint3.h ../Post/Views.h \
   ../Post/ColorTable.h ../Common/VertexArray.h ../Geo/SVector3.h \
   ../Geo/SPoint3.h ../Common/SmoothData.h ../Post/AdaptiveViews.h \
-  ../Common/GmshMatrix.h gl2ps.h ../Post/PView.h ../Post/AdaptiveViews.h \
-  ../Post/PViewData.h ../Post/PViewOptions.h ../Post/ColorTable.h
+  ../Common/GmshMatrix.h gl2ps.h
 Graph2D.o: Graph2D.cpp ../Common/GmshUI.h Draw.h ../DataStr/List.h \
   ../Geo/SBoundingBox3d.h ../Geo/SPoint3.h ../Post/PView.h \
   ../Common/VertexArray.h ../Geo/SVector3.h ../Geo/SPoint3.h \
-  ../Common/SmoothData.h ../Numeric/Numeric.h ../Post/AdaptiveViews.h \
-  ../Common/GmshMatrix.h ../Post/PViewData.h ../Post/PViewOptions.h \
-  ../Post/ColorTable.h gl2ps.h ../Common/Context.h
+  ../Common/SmoothData.h ../Numeric/Numeric.h ../Post/PViewData.h \
+  ../Post/PViewOptions.h ../Post/ColorTable.h gl2ps.h ../Common/Context.h
 gl2ps.o: gl2ps.cpp gl2ps.h
 gl2gif.o: gl2gif.cpp gl2gif.h PixelBuffer.h ../Common/Gmsh.h \
   ../Common/Message.h ../DataStr/Malloc.h ../DataStr/List.h \
diff --git a/Graphics/Post.cpp b/Graphics/Post.cpp
index 11067ce10fcbba8f65400ff844a594b83a088eb7..f5446d70c858eca203a33d4063f54110f075477a 100644
--- a/Graphics/Post.cpp
+++ b/Graphics/Post.cpp
@@ -1,4 +1,4 @@
-// $Id: Post.cpp,v 1.126 2007-08-31 09:18:16 geuzaine Exp $
+// $Id: Post.cpp,v 1.127 2007-09-08 21:26:04 geuzaine Exp $
 //
 // Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
 //
@@ -1141,7 +1141,7 @@ static int estimateNumPoints(PView *p)
   PViewData *data = p->getData();
   PViewOptions *opt = p->getOptions();
 
-  int heuristic = data->getNumPoints();
+  int heuristic = data->getNumElements(PViewData::Point);
   return heuristic + 10000;
 }
 
@@ -1150,7 +1150,7 @@ static int estimateNumLines(PView *p)
   PViewData *data = p->getData();
   PViewOptions *opt = p->getOptions();
 
-  int heuristic = data->getNumLines();
+  int heuristic = data->getNumElements(PViewData::Line);
   return heuristic + 10000;
 }
 
@@ -1159,12 +1159,12 @@ static int estimateNumTriangles(PView *p)
   PViewData *data = p->getData();
   PViewOptions *opt = p->getOptions();
 
-  int tris = data->getNumTriangles();
-  int quads = data->getNumQuadrangles();
-  int tets = data->getNumTetrahedra();
-  int prisms = data->getNumPrisms();
-  int pyrs = data->getNumPyramids();
-  int hexas = data->getNumHexahedra();
+  int tris = data->getNumElements(PViewData::Triangle);
+  int quads = data->getNumElements(PViewData::Quadrangle);
+  int tets = data->getNumElements(PViewData::Tetrahedron);
+  int prisms = data->getNumElements(PViewData::Prism);
+  int pyrs = data->getNumElements(PViewData::Pyramid);
+  int hexas = data->getNumElements(PViewData::Hexahedron);
 
   int heuristic = 0;
   if(opt->IntervalsType == PViewOptions::Iso)
@@ -1186,9 +1186,10 @@ static int estimateNumVectors(PView *p)
 
   int heuristic = data->getNumVectors();
   if(opt->Normals)
-    heuristic += data->getNumTriangles() + data->getNumQuadrangles();
+    heuristic += data->getNumElements(PViewData::Triangle) +
+      data->getNumElements(PViewData::Quadrangle);
   if(opt->Tangents)
-    heuristic += data->getNumLines();
+    heuristic += data->getNumElements(PViewData::Line);
 
   return heuristic + 1000;
 }
diff --git a/Parser/Makefile b/Parser/Makefile
index 92a3399b377a0ad4ab8cc7bb281ec4eaf8d8a756..1985569bc47bcdd745f023fb1e283fb15aeb3277 100644
--- a/Parser/Makefile
+++ b/Parser/Makefile
@@ -1,4 +1,4 @@
-# $Id: Makefile,v 1.124 2007-08-24 20:14:18 geuzaine Exp $
+# $Id: Makefile,v 1.125 2007-09-08 21:26:04 geuzaine Exp $
 #
 # Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
 #
@@ -133,12 +133,11 @@ OpenFile.o: OpenFile.cpp ../Common/Gmsh.h ../Common/Message.h \
   ../Geo/SBoundingBox3d.h Parser.h OpenFile.h ../Common/CommandLine.h \
   ../Post/Views.h ../Post/ColorTable.h ../Common/VertexArray.h \
   ../Post/AdaptiveViews.h ../Common/GmshMatrix.h ../Post/PView.h \
-  ../Post/AdaptiveViews.h ../Post/PViewData.h ../Post/PViewOptions.h \
-  ../Post/ColorTable.h ../Graphics/ReadImg.h ../Common/OS.h \
-  ../Mesh/HighOrder.h ../Common/GmshUI.h ../Graphics/Draw.h \
-  ../Graphics/SelectBuffer.h ../Fltk/GUI.h ../Fltk/Opengl_Window.h \
-  ../Fltk/Colorbar_Window.h ../Fltk/Popup_Button.h \
-  ../Fltk/SpherePosition_Widget.h
+  ../Post/PViewData.h ../Post/PViewOptions.h ../Post/ColorTable.h \
+  ../Graphics/ReadImg.h ../Common/OS.h ../Mesh/HighOrder.h \
+  ../Common/GmshUI.h ../Graphics/Draw.h ../Graphics/SelectBuffer.h \
+  ../Fltk/GUI.h ../Fltk/Opengl_Window.h ../Fltk/Colorbar_Window.h \
+  ../Fltk/Popup_Button.h ../Fltk/SpherePosition_Widget.h
 CreateFile.o: CreateFile.cpp ../Common/Gmsh.h ../Common/Message.h \
   ../DataStr/Malloc.h ../DataStr/List.h ../DataStr/Tree.h \
   ../DataStr/avl.h ../DataStr/Tools.h ../DataStr/List.h ../DataStr/Tree.h \
diff --git a/Parser/OpenFile.cpp b/Parser/OpenFile.cpp
index f4f11df682a137abd01ace083e95b522ea872e9a..3b5bd1b6dfdeed63ab04c1aa571fdfc48a7a048b 100644
--- a/Parser/OpenFile.cpp
+++ b/Parser/OpenFile.cpp
@@ -1,4 +1,4 @@
-// $Id: OpenFile.cpp,v 1.156 2007-08-31 17:30:06 geuzaine Exp $
+// $Id: OpenFile.cpp,v 1.157 2007-09-08 21:26:04 geuzaine Exp $
 //
 // Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
 //
@@ -389,9 +389,8 @@ int MergeFile(char *name, int warn_if_missing)
     }
     else if(!strncmp(header, "$PostFormat", 11) || 
 	    !strncmp(header, "$View", 5)) {
-#if 0 // test new post-pro
-      PView *p = new PView(false);
-      status = p->getData()->read(name);
+#if 1 // test new post-pro
+      status = PView::read(name);
 #else
       status = ReadView(name);
 #endif
diff --git a/Post/AdaptiveViews.h b/Post/AdaptiveViews.h
index a6ff5490f7cbb5fa3ab300cee85b501d8877d1b5..aea8958e27a02f708c7d9e8796eba4cda9b5fd80 100644
--- a/Post/AdaptiveViews.h
+++ b/Post/AdaptiveViews.h
@@ -21,6 +21,7 @@
 // Please report all bugs and problems to <gmsh@geuz.org>.
 
 #include <list>
+#include <set>
 #include "List.h"
 #include "GmshMatrix.h"
 
diff --git a/Post/Makefile b/Post/Makefile
index 2ee5233d9fc14b53b59f0e62bb53abac293cf928..248bd06f4cea3e218617233ae523d7eab696b75c 100644
--- a/Post/Makefile
+++ b/Post/Makefile
@@ -1,4 +1,4 @@
-# $Id: Makefile,v 1.11 2007-09-04 15:19:24 geuzaine Exp $
+# $Id: Makefile,v 1.12 2007-09-08 21:26:04 geuzaine Exp $
 #
 # Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
 #
@@ -63,32 +63,35 @@ depend:
 # DO NOT DELETE THIS LINE
 PView.o: PView.cpp PView.h ../Common/VertexArray.h ../Geo/SVector3.h \
   ../Geo/SPoint3.h ../Common/SmoothData.h ../Numeric/Numeric.h \
-  AdaptiveViews.h ../DataStr/List.h ../Common/GmshMatrix.h PViewData.h \
-  ../Geo/SBoundingBox3d.h ../Geo/SPoint3.h PViewOptions.h ColorTable.h \
-  PViewDataList.h ../Common/Message.h
+  PViewData.h ../Geo/SBoundingBox3d.h ../Geo/SPoint3.h PViewOptions.h \
+  ColorTable.h PViewDataList.h AdaptiveViews.h ../DataStr/List.h \
+  ../Common/GmshMatrix.h ../Common/Message.h
 PViewData.o: PViewData.cpp PViewData.h ../Geo/SBoundingBox3d.h \
   ../Geo/SPoint3.h
 PViewDataList.o: PViewDataList.cpp PViewDataList.h PViewData.h \
-  ../Geo/SBoundingBox3d.h ../Geo/SPoint3.h ../DataStr/List.h \
-  ../Numeric/Numeric.h ../Common/SmoothData.h ../Common/Context.h
+  ../Geo/SBoundingBox3d.h ../Geo/SPoint3.h AdaptiveViews.h \
+  ../DataStr/List.h ../Common/GmshMatrix.h ../Numeric/Numeric.h \
+  ../Common/SmoothData.h ../Common/Message.h ../Common/Context.h
 PViewDataListIO.o: PViewDataListIO.cpp PViewDataList.h PViewData.h \
-  ../Geo/SBoundingBox3d.h ../Geo/SPoint3.h ../DataStr/List.h \
-  ../Numeric/Numeric.h ../Common/Message.h ../Common/Context.h
+  ../Geo/SBoundingBox3d.h ../Geo/SPoint3.h AdaptiveViews.h \
+  ../DataStr/List.h ../Common/GmshMatrix.h ../Numeric/Numeric.h \
+  ../Common/Message.h ../Common/Context.h
 PViewDataGModel.o: PViewDataGModel.cpp PViewDataGModel.h PViewData.h \
   ../Geo/SBoundingBox3d.h ../Geo/SPoint3.h PViewDataList.h \
-  ../DataStr/List.h ../Geo/GModel.h ../Geo/GVertex.h ../Geo/GEntity.h \
-  ../Geo/Range.h ../Geo/SPoint3.h ../Geo/SBoundingBox3d.h \
-  ../Common/GmshDefines.h ../Geo/MVertex.h ../Geo/SPoint3.h \
-  ../Geo/GPoint.h ../Geo/SPoint2.h ../Geo/GEdge.h ../Geo/GEntity.h \
-  ../Geo/GVertex.h ../Geo/SVector3.h ../Geo/SPoint3.h ../Geo/SPoint3.h \
-  ../Geo/SPoint2.h ../Geo/MElement.h ../Geo/MVertex.h ../Geo/MEdge.h \
-  ../Geo/MVertex.h ../Geo/SVector3.h ../Common/Hash.h ../Geo/MFace.h \
-  ../Geo/MVertex.h ../Geo/SVector3.h ../Numeric/Numeric.h \
-  ../Common/Context.h ../Geo/ExtrudeParams.h ../Common/SmoothData.h \
-  ../Geo/GFace.h ../Geo/GPoint.h ../Geo/GEntity.h ../Geo/GEdgeLoop.h \
-  ../Geo/GEdge.h ../Geo/MElement.h ../Geo/SPoint2.h ../Geo/SVector3.h \
-  ../Geo/Pair.h ../Geo/ExtrudeParams.h ../Geo/GRegion.h ../Geo/GEntity.h \
-  ../Geo/MElement.h ../Geo/ExtrudeParams.h ../Geo/SBoundingBox3d.h
+  AdaptiveViews.h ../DataStr/List.h ../Common/GmshMatrix.h \
+  ../Geo/GModel.h ../Geo/GVertex.h ../Geo/GEntity.h ../Geo/Range.h \
+  ../Geo/SPoint3.h ../Geo/SBoundingBox3d.h ../Common/GmshDefines.h \
+  ../Geo/MVertex.h ../Geo/SPoint3.h ../Geo/GPoint.h ../Geo/SPoint2.h \
+  ../Geo/GEdge.h ../Geo/GEntity.h ../Geo/GVertex.h ../Geo/SVector3.h \
+  ../Geo/SPoint3.h ../Geo/SPoint3.h ../Geo/SPoint2.h ../Geo/MElement.h \
+  ../Geo/MVertex.h ../Geo/MEdge.h ../Geo/MVertex.h ../Geo/SVector3.h \
+  ../Common/Hash.h ../Geo/MFace.h ../Geo/MVertex.h ../Geo/SVector3.h \
+  ../Numeric/Numeric.h ../Common/Context.h ../Geo/ExtrudeParams.h \
+  ../Common/SmoothData.h ../Geo/GFace.h ../Geo/GPoint.h ../Geo/GEntity.h \
+  ../Geo/GEdgeLoop.h ../Geo/GEdge.h ../Geo/MElement.h ../Geo/SPoint2.h \
+  ../Geo/SVector3.h ../Geo/Pair.h ../Geo/ExtrudeParams.h ../Geo/GRegion.h \
+  ../Geo/GEntity.h ../Geo/MElement.h ../Geo/ExtrudeParams.h \
+  ../Geo/SBoundingBox3d.h
 PViewOptions.o: PViewOptions.cpp PViewOptions.h ColorTable.h \
   ../Geo/SBoundingBox3d.h ../Geo/SPoint3.h ../Common/Message.h
 Views.o: Views.cpp ../Common/Gmsh.h ../Common/Message.h \
diff --git a/Post/PView.cpp b/Post/PView.cpp
index b188cbcaef35ec91148a23c038c8f3c57df62d00..5035042e0e59e12ddab7f1a28abd9f5e74c1fe64 100644
--- a/Post/PView.cpp
+++ b/Post/PView.cpp
@@ -1,4 +1,4 @@
-// $Id: PView.cpp,v 1.6 2007-09-03 06:21:08 geuzaine Exp $
+// $Id: PView.cpp,v 1.7 2007-09-08 21:26:04 geuzaine Exp $
 //
 // Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
 //
@@ -32,7 +32,7 @@ std::vector<PView*> PView::list;
 PView::PView(bool allocate)
   : _changed(true), _aliasOf(0), _links(0), _eye(0., 0., 0.), 
     va_points(0), va_lines(0), va_triangles(0), va_vectors(0), 
-    normals(0), adaptive(0)
+    normals(0)
 {
   _num = ++_globalNum;
   _data = new PViewDataList(allocate);
@@ -45,7 +45,7 @@ PView::PView(bool allocate)
 PView::PView(PView *ref, bool copyOptions)
   : _changed(true), _links(0), _eye(0., 0., 0.), 
     va_points(0), va_lines(0), va_triangles(0), va_vectors(0), 
-    normals(0), adaptive(0)
+    normals(0)
 {
   _num = ++_globalNum;
   _aliasOf = ref->getNum();
@@ -67,7 +67,6 @@ PView::~PView()
   if(va_triangles) delete va_triangles;
   if(va_vectors) delete va_vectors;
   if(normals) delete normals;
-  if(adaptive) delete adaptive;
 
   if(_options) delete _options;
 
@@ -114,3 +113,140 @@ PView *PView::current()
   // return the last one for now
   return list.back();
 }
+
+bool PView::read(std::string filename, int fileIndex)
+{
+  FILE *fp = fopen(filename.c_str(), "rb");
+  if(!fp){
+    Msg(GERROR, "Unable to open file '%s'", filename.c_str());
+    return false;
+  }
+
+  char str[256];
+  double version;
+  int format, size, index = -1;
+
+  while(1) {
+
+    do {
+      if(!fgets(str, 256, fp) || feof(fp))
+        break;
+    } while(str[0] != '$');
+    
+    if(feof(fp))
+      break;
+
+    if(!strncmp(&str[1], "PostFormat", 10)) {
+
+      if(!fscanf(fp, "%lf %d %d\n", &version, &format, &size)){
+        Msg(GERROR, "Read error");
+        return false;
+      }
+      if(version < 1.0) {
+        Msg(GERROR, "Post-processing file too old (ver. %g < 1.0)", version);
+        return false;
+      }
+      if(size == sizeof(double))
+        Msg(DEBUG, "Data is in double precision format (size==%d)", size);
+      else {
+        Msg(GERROR, "Unknown data size (%d) in post-processing file", size);
+        return false;
+      }
+      if(format == 0)
+        format = LIST_FORMAT_ASCII;
+      else if(format == 1)
+        format = LIST_FORMAT_BINARY;
+      else {
+        Msg(GERROR, "Unknown format for view");
+        return false;
+      }
+
+    }
+    else if(!strncmp(&str[1], "View", 4)){
+
+      index++;
+      if(fileIndex < 0 || fileIndex == index){
+	PView *p = new PView(false);
+	PViewDataList *d = dynamic_cast<PViewDataList*>(p->getData());
+	if(!d || !d->read(fp, version, format, size)){
+	  Msg(GERROR, "Could not read data in list format");
+	  delete p;
+	  return false;
+	}
+	else{
+	  d->setFileName(filename);
+	  d->setFileIndex(index);
+	}
+      }
+
+    }
+
+    do {
+      if(!fgets(str, 256, fp) || feof(fp)){
+        Msg(GERROR, "Prematured end of file");
+	break;
+      }
+    } while(str[0] != '$');
+
+  }
+
+  fclose(fp);
+  return true;
+}
+
+void PView::combine(bool time, int how, bool remove)
+{
+  // time == true: combine the timesteps (oherwise combine the elements)
+  // how == 0: try to combine all visible views
+  //        1: try to combine all views
+  //        2: try to combine all views having identical names
+
+  std::vector<nameData> nds;
+  for(unsigned int i = 0; i < list.size(); i++) {
+    PView *p = list[i];
+    PViewData *data = p->getData();
+    if(how || p->getOptions()->Visible) {
+      nameData nd;
+      // this will lead to weird results if there are views named
+      // "__all__" or "__vis__" :-)
+      if(how == 2)
+	nd.name = data->getName();
+      else if(how == 1)
+	nd.name = "__all__";
+      else
+	nd.name = "__vis__";
+      unsigned int j = 0;
+      while(j < nds.size()){
+	if(nds[j].name == nd.name){
+	  nds[j].data.push_back(data);
+	  nds[j].indices.push_back(i);
+	  break;
+	}
+	j++;
+      }
+      if(j == nds.size()){
+	nd.data.push_back(data);
+	nd.indices.push_back(i);
+	nds.push_back(nd);
+      }
+    }
+  }
+
+  std::set<PView*> rm;
+  for(unsigned int i = 0; i < nds.size(); i++){
+    if(nds[i].data.size() > 1){
+      // there's potentially something to combine
+      PView *p = new PView(true);
+      PViewData *data = p->getData();
+      bool res = time ? data->combineTime(nds[i]): data->combineSpace(nds[i]);
+      if(res)
+	for(unsigned int j = 0; j < nds[i].indices.size(); j++)
+	  rm.insert(list[nds[i].indices[j]]);
+      else
+	delete p;
+    }
+  }
+  if(remove)
+    for(std::set<PView*>::iterator it = rm.begin(); it != rm.end(); it++)
+      delete *it;
+}
diff --git a/Post/PViewData.cpp b/Post/PViewData.cpp
index 3e8044a1cb69742cfc9bd1b2f910fbd7dbd408ee..7b5be6a0bfc18301e785ccbfad838ef747db2b71 100644
--- a/Post/PViewData.cpp
+++ b/Post/PViewData.cpp
@@ -1,4 +1,4 @@
-// $Id: PViewData.cpp,v 1.9 2007-09-01 16:05:43 geuzaine Exp $
+// $Id: PViewData.cpp,v 1.10 2007-09-08 21:26:04 geuzaine Exp $
 //
 // Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
 //
@@ -23,3 +23,15 @@
 // 
 
 #include "PViewData.h"
+
+PViewData::PViewData()
+  : _dirty(true), _name(""), _filename(""), _fileIndex(0)
+{
+}
+
+bool PViewData::empty()
+{
+  return (!getNumElements() && 
+	  !getNumStrings2D() &&
+	  !getNumStrings3D());
+}
diff --git a/Post/PViewData.h b/Post/PViewData.h
index 0617aba8f954b00677a5eb71616df6cb8a326a3d..ec23f859940b47fdb3c9c9215dface8c28e2105d 100644
--- a/Post/PViewData.h
+++ b/Post/PViewData.h
@@ -21,8 +21,12 @@
 // Please report all bugs and problems to <gmsh@geuz.org>.
 
 #include <string>
+#include <vector>
 #include "SBoundingBox3d.h"
 
+class GMSH_Post_Plugin;
+class nameData;
+
 // abstract interface to post-processing view data
 class PViewData {
  private:
@@ -32,17 +36,31 @@ class PViewData {
   std::string _name;
   // name of the file the data was loaded from
   std::string _filename;
+  // index of the view in the file
+  int _fileIndex;
  public:
-  PViewData() : _dirty(true) {}
+  enum ElementType {
+    Point=1,
+    Line=2,
+    Triangle=3,
+    Quadrangle=4,
+    Tetrahedron=5,
+    Hexahedron=6,
+    Prism=7,
+    Pyramid=8
+  };
+  PViewData();
   virtual ~PViewData(){}
   virtual bool getDirty(){ return _dirty; }
   virtual void setDirty(bool val){ _dirty = val; }
   virtual void finalize(){ _dirty = false; }
-  virtual int getNumTimeSteps() = 0;
   virtual std::string getName(){ return _name; }
   virtual void setName(std::string val){ _name = val; }
   virtual std::string getFileName(){ return _filename; }
   virtual void setFileName(std::string val){ _filename = val; }
+  virtual int getFileIndex(){ return _fileIndex; }
+  virtual void setFileIndex(int val){ _fileIndex = val; }
+  virtual int getNumTimeSteps() = 0;
   virtual double getTime(int step){ return 0.; }
   virtual double getMin(int step=-1) = 0;
   virtual double getMax(int step=-1) = 0;
@@ -50,15 +68,7 @@ class PViewData {
   virtual int getNumScalars(){ return 0; }
   virtual int getNumVectors(){ return 0; }
   virtual int getNumTensors(){ return 0; }
-  virtual int getNumPoints(){ return 0; }
-  virtual int getNumLines(){ return 0; }
-  virtual int getNumTriangles(){ return 0; }
-  virtual int getNumQuadrangles(){ return 0; }
-  virtual int getNumTetrahedra(){ return 0; }
-  virtual int getNumHexahedra(){ return 0; }
-  virtual int getNumPrisms(){ return 0; }
-  virtual int getNumPyramids(){ return 0; }
-  virtual int getNumElements() = 0;
+  virtual int getNumElements(int type=0) = 0;
   virtual int getDimension(int ele) = 0;
   virtual int getEntity(int ele){ return 0; }
   virtual int getNumNodes(int ele) = 0;
@@ -72,13 +82,34 @@ class PViewData {
 			   double &x, double &y, double &style){}
   virtual void getString3D(int i, int step, std::string &str, 
 			   double &x, double &y, double &z, double &style){}
-  virtual bool read(std::string name){ return false; }
+  virtual bool empty();
+  virtual void smooth(){}
+  virtual bool combineTime(nameData &nd){ return false; }
+  virtual bool combineSpace(nameData &nd){ return false; }
+  virtual void setGlobalResolutionLevel(int level){}
+  virtual void setAdaptiveResolutionLevel(int level, GMSH_Post_Plugin *plugin=0){}
+
+  // I/O routines
   virtual bool writePOS(std::string name, bool binary=false, bool parsed=true,
 			bool append=false){ return false; }
   virtual bool writeSTL(std::string name){ return false; }
   virtual bool writeTXT(std::string name){ return false; }
   virtual bool writeMSH(std::string name){ return false; }
+};
 
+class nameData{
+ public:
+  std::string name;
+  std::vector<int> indices;
+  std::vector<PViewData*> data;
+};
+
+class nameDataLessThan{
+ public:
+  bool operator()(const nameData &n1, const nameData &n2) const
+  {
+    return n1.name < n2.name;
+  }
 };
 
 #endif
diff --git a/Post/PViewDataGModel.cpp b/Post/PViewDataGModel.cpp
index 3fc2cf502948007773562dcf11ca876dce32ce6e..9d03368cd01c038be20d7bef658ca0386dbccd7b 100644
--- a/Post/PViewDataGModel.cpp
+++ b/Post/PViewDataGModel.cpp
@@ -1,4 +1,4 @@
-// $Id: PViewDataGModel.cpp,v 1.2 2007-09-02 21:05:20 geuzaine Exp $
+// $Id: PViewDataGModel.cpp,v 1.3 2007-09-08 21:26:05 geuzaine Exp $
 //
 // Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
 //
@@ -24,11 +24,6 @@
 
 #include "PViewDataGModel.h"
 
-bool PViewDataGModel::read(std::string name)
-{
-  // model->read();
-}
-
 bool PViewDataGModel::writePOS(std::string name, bool binary, bool parsed,
 			       bool append)
 {
diff --git a/Post/PViewDataGModel.h b/Post/PViewDataGModel.h
index f2864fc7b882adff432fb5411166fa7ac91d4b84..5f81f582cc473e8ed239d8c05631606f66d4351b 100644
--- a/Post/PViewDataGModel.h
+++ b/Post/PViewDataGModel.h
@@ -37,14 +37,15 @@ class PViewDataGModel : public PViewData {
   double getMin(int step=-1){ return 0.; }
   double getMax(int step=-1){ return 1.; }
   SBoundingBox3d getBoundingBox(){ return SBoundingBox3d(); }
-  int getNumElements(){ return _model->numElements(); }
+  int getNumElements(int type=0){ return _model->numElements(); }
   int getDimension(int ele){ return 0; }
   int getNumNodes(int ele){ return 0; }
   void getNode(int ele, int nod, double &x, double &y, double &z){}
   int getNumComponents(int ele){ return 1; }
   void getValue(int ele, int node, int comp, int step, double &val){}
   int getNumEdges(int ele){ return 0; }
-  bool read(std::string name);
+
+  // I/O routines
   bool writePOS(std::string name, bool binary=false, bool parsed=true,
 		bool append=false);
   bool writeSTL(std::string name);
diff --git a/Post/PViewDataList.cpp b/Post/PViewDataList.cpp
index 63ed65053378c866b11bcb9d1a8019e57b6bca39..c8b5d481a041a9aa8f132e19116e4d1e6a9a30ee 100644
--- a/Post/PViewDataList.cpp
+++ b/Post/PViewDataList.cpp
@@ -1,4 +1,4 @@
-// $Id: PViewDataList.cpp,v 1.2 2007-09-02 21:05:20 geuzaine Exp $
+// $Id: PViewDataList.cpp,v 1.3 2007-09-08 21:26:05 geuzaine Exp $
 //
 // Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
 //
@@ -25,6 +25,7 @@
 #include "PViewDataList.h"
 #include "Numeric.h"
 #include "SmoothData.h"
+#include "Message.h"
 #include "Context.h"
 
 extern Context_T CTX;
@@ -47,7 +48,7 @@ PViewDataList::PViewDataList(bool allocate)
     NbSI2(0), SI2(0), NbVI2(0), VI2(0), NbTI2(0), TI2(0),
     NbSY(0), SY(0), NbVY(0), VY(0), NbTY(0), TY(0),
     NbSY2(0), SY2(0), NbVY2(0), VY2(0), NbTY2(0), TY2(0),
-    NbT2(0), T2D(0), T2C(0), NbT3(0), T3D(0), T3C(0),
+    NbT2(0), T2D(0), T2C(0), NbT3(0), T3D(0), T3C(0), adaptive(0), 
     _lastElement(-1), _lastDimension(-1), _lastNumNodes(-1), 
     _lastNumComponents(-1), _lastNumEdges(-1), _lastXYZ(0), _lastVal(0)
 {
@@ -69,8 +70,6 @@ PViewDataList::PViewDataList(bool allocate)
     T2C = List_Create(1, 100, sizeof(char));
     T3D = List_Create(1, 100, sizeof(double));
     T3C = List_Create(1, 100, sizeof(char));
-    Grains = new std::map<int, List_T*>;
-    DisplayListsOfGrains= new std::map<int, int>;
   }
 }
 
@@ -87,8 +86,7 @@ PViewDataList::~PViewDataList()
   List_Delete(SY); List_Delete(VY); List_Delete(TY);
   List_Delete(T2D); List_Delete(T2C);
   List_Delete(T3D); List_Delete(T3C);
-  if(Grains) delete Grains;
-  if(DisplayListsOfGrains) delete DisplayListsOfGrains;
+  if(adaptive) delete adaptive;
 }
 
 void PViewDataList::finalize()
@@ -140,6 +138,40 @@ void PViewDataList::finalize()
   setDirty(false);
 }
 
+int PViewDataList::getNumScalars()
+{ 
+  return NbSP + NbSL + NbST + NbSQ + NbSS + NbSH + NbSI + NbSY; 
+}
+
+int PViewDataList::getNumVectors()
+{
+  return NbVP + NbVL + NbVT + NbVQ + NbVS + NbVH + NbVI + NbVY; 
+}
+
+int PViewDataList::getNumTensors()
+{ 
+  return NbTP + NbTL + NbTT + NbTQ + NbTS + NbTH + NbTI + NbTY; 
+}
+
+int PViewDataList::getNumElements(int type)
+{
+  if(type){
+    switch(type){
+    case Point: return NbSP + NbVP + NbTP;
+    case Line: return NbSL + NbVL + NbTL;
+    case Triangle: return NbST + NbVT + NbTT;
+    case Quadrangle: return NbSQ + NbVQ + NbTQ;
+    case Tetrahedron: return NbSS + NbVS + NbTS;
+    case Hexahedron: return NbSH + NbVH + NbTH;
+    case Prism: return NbSI + NbVI + NbTI;
+    case Pyramid: return NbSY + NbVY + NbTY;
+    default: Msg(GERROR, "Unknown element type"); return 0;
+    }
+  }
+
+  return getNumScalars() + getNumVectors() + getNumTensors();
+}
+
 double PViewDataList::getTime(int step)
 {
   if(step < 0 || step >= List_Nbr(Time)) return 0.;
@@ -586,3 +618,268 @@ void PViewDataList::smooth()
 
   xyzv::eps = old_eps;  
 }
+
+bool PViewDataList::combineSpace(nameData &nd)
+{
+  // sanity checks
+  if(nd.data.size() < 2) return false;
+  int ts = nd.data[0]->getNumTimeSteps();
+  for(unsigned int i = 1; i < nd.data.size(); i++) {
+    if(nd.data[i]->getNumTimeSteps() != ts){
+      Msg(GERROR, "Cannot combine views having different number of time steps");
+      return false;
+    }
+  }
+
+  for(unsigned int i = 0; i < nd.data.size(); i++) {
+    PViewDataList *l = dynamic_cast<PViewDataList*>(nd.data[i]);
+    if(!l){
+      Msg(GERROR, "Cannot combine hybrid data");
+      return false;
+    }
+    // merge elememts
+    List_Merge(l->SP, SP); NbSP += l->NbSP; List_Merge(l->VP, VP); NbVP += l->NbVP;
+    List_Merge(l->TP, TP); NbTP += l->NbTP; List_Merge(l->SL, SL); NbSL += l->NbSL;
+    List_Merge(l->VL, VL); NbVL += l->NbVL; List_Merge(l->TL, TL); NbTL += l->NbTL;
+    List_Merge(l->ST, ST); NbST += l->NbST; List_Merge(l->VT, VT); NbVT += l->NbVT;
+    List_Merge(l->TT, TT); NbTT += l->NbTT; List_Merge(l->SQ, SQ); NbSQ += l->NbSQ;
+    List_Merge(l->VQ, VQ); NbVQ += l->NbVQ; List_Merge(l->TQ, TQ); NbTQ += l->NbTQ;
+    List_Merge(l->SS, SS); NbSS += l->NbSS; List_Merge(l->VS, VS); NbVS += l->NbVS;
+    List_Merge(l->TS, TS); NbTS += l->NbTS; List_Merge(l->SH, SH); NbSH += l->NbSH;
+    List_Merge(l->VH, VH); NbVH += l->NbVH; List_Merge(l->TH, TH); NbTH += l->NbTH;
+    List_Merge(l->SI, SI); NbSI += l->NbSI; List_Merge(l->VI, VI); NbVI += l->NbVI;
+    List_Merge(l->TI, TI); NbTI += l->NbTI; List_Merge(l->SY, SY); NbSY += l->NbSY;
+    List_Merge(l->VY, VY); NbVY += l->NbVY; List_Merge(l->TY, TY); NbTY += l->NbTY;
+
+    // merge strings
+    for(int i = 0; i < List_Nbr(l->T2D); i += 4){
+      List_Add(T2D, List_Pointer(l->T2D, i));
+      List_Add(T2D, List_Pointer(l->T2D, i + 1));
+      List_Add(T2D, List_Pointer(l->T2D, i + 2)); 
+      double d = List_Nbr(T2C);
+      List_Add(T2D, &d);
+      double beg, end;
+      List_Read(l->T2D, i + 3, &beg); 
+      if(i > List_Nbr(l->T2D) - 8)
+	end = (double)List_Nbr(l->T2C);
+      else
+	List_Read(l->T2D, i + 3 + 4, &end); 
+      char *c = (char*)List_Pointer(l->T2C, (int)beg);
+      for(int j = 0; j < (int)(end - beg); j++)
+	List_Add(T2C, &c[j]); 
+      NbT2++;
+    }
+    for(int i = 0; i < List_Nbr(l->T3D); i += 5){
+      List_Add(T3D, List_Pointer(l->T3D, i));
+      List_Add(T3D, List_Pointer(l->T3D, i + 1));
+      List_Add(T3D, List_Pointer(l->T3D, i + 2)); 
+      List_Add(T3D, List_Pointer(l->T3D, i + 3)); 
+      double d = List_Nbr(T3C);
+      List_Add(T3D, &d);
+      double beg, end;
+      List_Read(l->T3D, i + 4, &beg); 
+      if(i > List_Nbr(l->T3D) - 10)
+	end = (double)List_Nbr(l->T3C);
+      else
+	List_Read(l->T3D, i + 4 + 5, &end); 
+      char *c = (char*)List_Pointer(l->T3C, (int)beg);
+      for(int j = 0; j < (int)(end-beg); j++)
+	List_Add(T3C, &c[j]); 
+      NbT3++;
+    }
+  }
+  
+  std::string tmp;
+  if(nd.name == "__all__")
+    tmp = "all";
+  else if(nd.name == "__vis__")
+    tmp = "visible";
+  else
+    tmp = nd.name;
+  char name[256];
+  sprintf(name, "%s_Combine", tmp.c_str());
+
+  setName(name);
+  setFileName(std::string(name) + ".pos");
+  finalize();
+}
+
+void PViewDataList::getRawData(int type, List_T **l, int **ne, int *nc, int *nn)
+{
+  switch(type){
+  case 0 : *l = SP; *ne = &NbSP; *nc = 1; *nn = 1; break;
+  case 1 : *l = VP; *ne = &NbVP; *nc = 3; *nn = 1; break;
+  case 2 : *l = TP; *ne = &NbTP; *nc = 9; *nn = 1; break;
+  case 3 : *l = SL; *ne = &NbSL; *nc = 1; *nn = 2; break;
+  case 4 : *l = VL; *ne = &NbVL; *nc = 3; *nn = 2; break;
+  case 5 : *l = TL; *ne = &NbTL; *nc = 9; *nn = 2; break;
+  case 6 : *l = ST; *ne = &NbST; *nc = 1; *nn = 3; break;
+  case 7 : *l = VT; *ne = &NbVT; *nc = 3; *nn = 3; break;
+  case 8 : *l = TT; *ne = &NbTT; *nc = 9; *nn = 3; break;
+  case 9 : *l = SQ; *ne = &NbSQ; *nc = 1; *nn = 4; break;
+  case 10: *l = VQ; *ne = &NbVQ; *nc = 3; *nn = 4; break;
+  case 11: *l = TQ; *ne = &NbTQ; *nc = 9; *nn = 4; break;
+  case 12: *l = SS; *ne = &NbSS; *nc = 1; *nn = 4; break;
+  case 13: *l = VS; *ne = &NbVS; *nc = 3; *nn = 4; break;
+  case 14: *l = TS; *ne = &NbTS; *nc = 9; *nn = 4; break;
+  case 15: *l = SH; *ne = &NbSH; *nc = 1; *nn = 8; break;
+  case 16: *l = VH; *ne = &NbVH; *nc = 3; *nn = 8; break;
+  case 17: *l = TH; *ne = &NbTH; *nc = 9; *nn = 8; break;
+  case 18: *l = SI; *ne = &NbSI; *nc = 1; *nn = 6; break;
+  case 19: *l = VI; *ne = &NbVI; *nc = 3; *nn = 6; break;
+  case 20: *l = TI; *ne = &NbTI; *nc = 9; *nn = 6; break;
+  case 21: *l = SY; *ne = &NbSY; *nc = 1; *nn = 5; break;
+  case 22: *l = VY; *ne = &NbVY; *nc = 3; *nn = 5; break;
+  case 23: *l = TY; *ne = &NbTY; *nc = 9; *nn = 5; break;
+  default: Msg(GERROR, "Wrong type in PViewDataList"); break;
+  }
+}
+
+bool PViewDataList::combineTime(nameData &nd)
+{
+  // sanity checks
+  if(nd.data.size() < 2) return false;
+  std::vector<PViewDataList*> data(nd.data.size());
+  for(unsigned int i = 0; i < nd.data.size(); i++){
+    data[i] = dynamic_cast<PViewDataList*>(nd.data[i]);
+    if(!data[i]){
+      Msg(GERROR, "Cannot combine hybrid data");
+      return false;
+    }
+  }
+
+  int *nbe=0, *nbe2=0, nbn, nbn2, nbc, nbc2;
+  List_T *list=0, *list2=0;
+  
+  // use the first data set as the reference
+  for(int i = 0; i < 24; i++){
+    getRawData(i, &list, &nbe, &nbc, &nbn);
+    data[0]->getRawData(i, &list2, &nbe2, &nbc2, &nbn2);
+    *nbe = *nbe2;
+  }
+  NbT2 = data[0]->NbT2;
+  NbT3 = data[0]->NbT3;
+
+  // merge values for all element types
+  for(int i = 0; i < 24; i++){
+    getRawData(i, &list, &nbe, &nbc, &nbn);
+    for(int j = 0; j < *nbe; j++){
+      for(unsigned int k = 0; k < data.size(); k++){
+	data[k]->getRawData(i, &list2, &nbe2, &nbc2, &nbn2);
+	if(*nbe && *nbe == *nbe2){
+	  int nb2 = List_Nbr(list2) / *nbe2;
+	  if(!k){ 
+	    // copy coordinates of elm j (we are always here as
+	    // expected, since the ref view is the first one)
+	    for(int l = 0; l < 3 * nbn2; l++)
+	      List_Add(list, List_Pointer(list2, j * nb2 + l));
+	  }
+	  // copy values of elm j
+	  for(int l = 0; l < data[k]->getNumTimeSteps() * nbc2 * nbn2; l++)
+	    List_Add(list, List_Pointer(list2, j * nb2 + 3 * nbn2 + l));
+	}
+      }
+    }
+  }
+
+  // merge 2d strings
+  for(int j = 0; j < NbT2; j++){
+    for(unsigned int k = 0; k < data.size(); k++){
+      if(NbT2 == data[k]->NbT2){
+	if(!k){
+	  // copy coordinates 
+	  List_Add(T2D, List_Pointer(data[k]->T2D, j * 4));
+	  List_Add(T2D, List_Pointer(data[k]->T2D, j * 4 + 1));
+	  List_Add(T2D, List_Pointer(data[k]->T2D, j * 4 + 2));
+	  // index
+	  double d = List_Nbr(T2C);
+	  List_Add(T2D, &d);
+	}
+	// copy char values
+	double beg, end;
+	List_Read(data[k]->T2D, j * 4 + 3, &beg);
+	if(j == NbT2 - 1)
+	  end = (double)List_Nbr(data[k]->T2C);
+	else
+	  List_Read(data[k]->T2D, j * 4 + 4 + 3, &end);
+	char *c = (char*)List_Pointer(data[k]->T2C, (int)beg);
+	for(int l = 0; l < (int)(end - beg); l++)
+	  List_Add(T2C, &c[l]);
+      }
+    }
+  }
+
+  // merge 3d strings
+  for(int j = 0; j < NbT3; j++){
+    for(unsigned int k = 0; k < data.size(); k++){
+      if(NbT3 == data[k]->NbT3){
+	if(!k){
+	  // copy coordinates 
+	  List_Add(T3D, List_Pointer(data[k]->T3D, j * 5));
+	  List_Add(T3D, List_Pointer(data[k]->T3D, j * 5 + 1));
+	  List_Add(T3D, List_Pointer(data[k]->T3D, j * 5 + 2));
+	  List_Add(T3D, List_Pointer(data[k]->T3D, j * 5 + 3));
+	  // index
+	  double d = List_Nbr(T3C);
+	  List_Add(T3D, &d);
+	}
+	// copy char values
+	double beg, end;
+	List_Read(data[k]->T3D, j * 5 + 4, &beg);
+	if(j == NbT3 - 1)
+	  end = (double)List_Nbr(data[k]->T3C);
+	else
+	  List_Read(data[k]->T3D, j * 5 + 5 + 4, &end);
+	char *c = (char*)List_Pointer(data[k]->T3C, (int)beg);
+	for(int l = 0; l < (int)(end - beg); l++)
+	  List_Add(T3C, &c[l]);
+      }
+    }
+  }
+
+  // create the time data
+  for(unsigned int i = 0; i < data.size(); i++)
+    List_Merge(data[i]->Time, Time);
+  
+  // if all the time values are the same, it probably means that the
+  // original views didn't have any time data: then we'll just use
+  // time step values
+  if(List_Nbr(Time)){
+    double t0, ti;
+    List_Read(Time, 0, &t0);
+    bool allTheSame = true;
+    for(int i = 1; i < List_Nbr(Time); i++){
+      List_Read(Time, i, &ti);
+      if(ti != t0){
+	allTheSame = false;
+	break;
+      }
+    }
+    if(allTheSame) List_Reset(Time);
+  }
+
+  std::string tmp;
+  if(nd.name == "__all__")
+    tmp = "all";
+  else if(nd.name == "__vis__")
+    tmp = "visible";
+  else
+    tmp = nd.name;
+  char name[256];
+  sprintf(name, "%s_Combine", tmp.c_str());
+
+  setName(name);
+  setFileName(std::string(name) + ".pos");
+  finalize();
+}
+
+void PViewDataList::setGlobalResolutionLevel(int level)
+{
+  //if(adaptive) adaptive->setGlobalResolutionLevel(this, level);
+}
+
+void PViewDataList::setAdaptiveResolutionLevel(int level, GMSH_Post_Plugin *plugin)
+{
+  //if(adaptive) adaptive->setAdaptiveResolutionLevel(this, level, plugin);
+}
+
diff --git a/Post/PViewDataList.h b/Post/PViewDataList.h
index eab7173e098cf92eefd3b352f0e571bb18158b2f..db580aa5f77d79fb7453090603f5fab6909b9f6c 100644
--- a/Post/PViewDataList.h
+++ b/Post/PViewDataList.h
@@ -25,6 +25,7 @@
 #include <string>
 #include "PViewData.h"
 #include "SBoundingBox3d.h"
+#include "AdaptiveViews.h"
 #include "List.h"
 
 #define VAL_INF 1.e200
@@ -60,6 +61,7 @@ class PViewDataList : public PViewData {
   List_T *T2D, *T2C, *T3D, *T3C; // 2D and 3D text strings
   std::map<int, List_T*> *Grains; // For LMGC90, grains shapes
   std::map<int, int> *DisplayListsOfGrains; // For LMGC90, grains shapes
+  Adaptive_Post_View *adaptive;
  private:
   int _index[24];
   int _lastElement, _lastDimension;
@@ -82,23 +84,10 @@ class PViewDataList : public PViewData {
   double getMin(int step=-1);
   double getMax(int step=-1);
   SBoundingBox3d getBoundingBox(){ return BBox; }
-  int getNumScalars(){ return NbSP + NbSL + NbST + NbSQ + NbSS + NbSH + NbSI + NbSY; }
-  int getNumVectors(){ return NbVP + NbVL + NbVT + NbVQ + NbVS + NbVH + NbVI + NbVY; }
-  int getNumTensors(){ return NbTP + NbTL + NbTT + NbTQ + NbTS + NbTH + NbTI + NbTY; }
-  int getNumPoints(){ return NbSP + NbVP + NbTP; }
-  int getNumLines(){ return NbSL + NbVL + NbTL; }
-  int getNumTriangles(){ return NbST + NbVT + NbTT; }
-  int getNumQuadrangles(){ return NbSQ + NbVQ + NbTQ; }
-  int getNumTetrahedra(){ return NbSS + NbVS + NbTS; }
-  int getNumHexahedra(){ return NbSH + NbVH + NbTH; }
-  int getNumPrisms(){ return NbSI + NbVI + NbTI; }
-  int getNumPyramids(){ return NbSY + NbVY + NbTY; }
-  int getNumElements()
-  {
-    return getNumPoints() + getNumLines() + getNumTriangles() + 
-      getNumQuadrangles() + getNumTetrahedra() + getNumHexahedra() + 
-      getNumPrisms() + getNumPyramids();
-  }
+  int getNumScalars();
+  int getNumVectors();
+  int getNumTensors();
+  int getNumElements(int type=0);
   int getDimension(int ele);
   int getNumNodes(int ele);
   void getNode(int ele, int nod, double &x, double &y, double &z);
@@ -112,7 +101,16 @@ class PViewDataList : public PViewData {
   void getString3D(int i, int step, std::string &str, 
 		   double &x, double &y, double &z, double &style);
   void smooth();
-  bool read(std::string name);
+  bool combineTime(nameData &nd);
+  bool combineSpace(nameData &nd);
+  void setGlobalResolutionLevel(int level);
+  void setAdaptiveResolutionLevel(int level, GMSH_Post_Plugin *plugin=0);
+
+  // specific to list-based data sets
+  void getRawData(int type, List_T **l, int **ne, int *nc, int *nn);
+
+  // I/O routines
+  bool read(FILE *fp, double version, int format, int size);
   bool writePOS(std::string name, bool binary=false, bool parsed=true,
 		bool append=false);
   bool writeSTL(std::string name);
diff --git a/Post/PViewDataListIO.cpp b/Post/PViewDataListIO.cpp
index 82be6bdb9bd5d2a54fb8a45dfce067d54b6b543f..511ae6cc08940831f153316cb9e63b848a3d55b8 100644
--- a/Post/PViewDataListIO.cpp
+++ b/Post/PViewDataListIO.cpp
@@ -1,4 +1,4 @@
-// $Id: PViewDataListIO.cpp,v 1.2 2007-09-02 21:05:20 geuzaine Exp $
+// $Id: PViewDataListIO.cpp,v 1.3 2007-09-08 21:26:05 geuzaine Exp $
 //
 // Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
 //
@@ -30,284 +30,225 @@
 
 extern Context_T CTX;
 
-bool PViewDataList::read(std::string filename)
+bool PViewDataList::read(FILE *fp, double version, int format, int size)
 {
-  FILE *fp = fopen(filename.c_str(), "rb");
-  if(!fp){
-    Msg(GERROR, "Unable to open file '%s'", filename.c_str());
+  char name[256];
+  int t2l, t3l;
+
+  if(version <= 1.0) {
+    Msg(DEBUG, "Detected post-processing view format <= 1.0");
+    if(!fscanf(fp, "%s %d %d %d %d %d %d %d %d %d %d %d %d %d\n",
+	       name, &NbTimeStep, &NbSP, &NbVP, &NbTP, &NbSL, &NbVL, &NbTL,
+	       &NbST, &NbVT, &NbTT, &NbSS, &NbVS, &NbTS)){
+      Msg(GERROR, "Read error");
+      return false;
+    }
+    NbT2 = t2l = NbT3 = t3l = 0;
+  }
+  else if(version == 1.1) {
+    Msg(DEBUG, "Detected post-processing view format 1.1");
+    if(!fscanf(fp, "%s %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d\n",
+	       name, &NbTimeStep, &NbSP, &NbVP, &NbTP, &NbSL, &NbVL, &NbTL, 
+	       &NbST, &NbVT, &NbTT, &NbSS, &NbVS, &NbTS, &NbT2, &t2l, &NbT3,
+	       &t3l)){
+      Msg(GERROR, "Read error");
+      return false;
+    }
+  }
+  else if(version == 1.2 || version == 1.3) {
+    Msg(DEBUG, "Detected post-processing view format %g", version);
+    if(!fscanf(fp, "%s %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d "
+	       "%d %d %d %d %d %d %d %d %d %d %d %d %d\n",
+	       name, &NbTimeStep, &NbSP, &NbVP, &NbTP, &NbSL, &NbVL, &NbTL,
+	       &NbST, &NbVT, &NbTT, &NbSQ, &NbVQ, &NbTQ, &NbSS, &NbVS, &NbTS, 
+	       &NbSH, &NbVH, &NbTH, &NbSI, &NbVI, &NbTI, &NbSY, &NbVY, &NbTY,
+	       &NbT2, &t2l, &NbT3, &t3l)){
+      Msg(GERROR, "Read error");
+      return false;
+    }
+  }
+  else if(version == 1.4) {
+    Msg(DEBUG, "Detected post-processing view format 1.4");
+    if(!fscanf(fp, "%s %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d "
+	       "%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d "
+	       "%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d\n",
+	       name, &NbTimeStep, &NbSP, &NbVP, &NbTP, &NbSL, &NbVL, &NbTL,
+	       &NbST, &NbVT, &NbTT, &NbSQ, &NbVQ, &NbTQ, &NbSS, &NbVS, &NbTS, 
+	       &NbSH, &NbVH, &NbTH, &NbSI, &NbVI, &NbTI, &NbSY, &NbVY, &NbTY,
+	       &NbSL2, &NbVL2, &NbTL2, &NbST2, &NbVT2, &NbTT2, &NbSQ2, &NbVQ2, 
+	       &NbTQ2, &NbSS2, &NbVS2, &NbTS2, &NbSH2, &NbVH2, &NbTH2, &NbSI2, 
+	       &NbVI2, &NbTI2, &NbSY2, &NbVY2, &NbTY2, &NbT2, &t2l, &NbT3, &t3l)){
+      Msg(GERROR, "Read error");
+      return false;
+    }
+  }
+  else {
+    Msg(GERROR, "Unknown post-processing file format (version %g)", version);
     return false;
   }
-
-  char str[256], name[256];
-  int format, size, swap, t2l, t3l;
-  double version;
-
-  while(1) {
-
-    do {
-      if(!fgets(str, 256, fp) || feof(fp))
-        break;
-    } while(str[0] != '$');
-    
-    if(feof(fp))
-      break;
-
-    if(!strncmp(&str[1], "PostFormat", 10)) {
-      if(!fscanf(fp, "%lf %d %d\n", &version, &format, &size)){
-        Msg(GERROR, "Read error");
-        return false;
-      }
-      if(version < 1.0) {
-        Msg(GERROR, "Post-processing file too old (ver. %g < 1.0)", version);
-        return false;
-      }
-      if(size == sizeof(double))
-        Msg(DEBUG, "Data is in double precision format (size==%d)", size);
-      else {
-        Msg(GERROR, "Unknown data size (%d) in post-processing file", size);
-        return false;
-      }
-      if(format == 0)
-        format = LIST_FORMAT_ASCII;
-      else if(format == 1)
-        format = LIST_FORMAT_BINARY;
-      else {
-        Msg(GERROR, "Unknown format for view");
-        return false;
-      }
+  
+  for(int i = 0; i < (int)strlen(name); i++)
+    if(name[i] == '^')
+      name[i] = ' ';
+  
+  int swap = 0;
+  if(format == LIST_FORMAT_BINARY) {
+    int testone;
+    if(!fread(&testone, sizeof(int), 1, fp)){
+      Msg(GERROR, "Read error");
+      return false;
     }
-
-    if(!strncmp(&str[1], "View", 4)) {
-      if(version <= 1.0) {
-        Msg(DEBUG, "Detected post-processing view format <= 1.0");
-        if(!fscanf(fp, "%s %d %d %d %d %d %d %d %d %d %d %d %d %d\n",
-		   name, &NbTimeStep,
-		   &NbSP, &NbVP, &NbTP, &NbSL, &NbVL, &NbTL,
-		   &NbST, &NbVT, &NbTT, &NbSS, &NbVS, &NbTS)){
-	  Msg(GERROR, "Read error");
-	  return false;
-	}
-        NbT2 = t2l = NbT3 = t3l = 0;
-      }
-      else if(version == 1.1) {
-        Msg(DEBUG, "Detected post-processing view format 1.1");
-        if(!fscanf(fp, "%s %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d\n",
-		   name, &NbTimeStep, &NbSP, &NbVP, &NbTP, &NbSL,
-		   &NbVL, &NbTL, &NbST, &NbVT, &NbTT, &NbSS,
-		   &NbVS, &NbTS, &NbT2, &t2l, &NbT3, &t3l)){
-	  Msg(GERROR, "Read error");
-	  return false;
-	}
-      }
-      else if(version == 1.2 || version == 1.3) {
-        Msg(DEBUG, "Detected post-processing view format %g", version);
-        if(!fscanf(fp, "%s %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d "
-		   "%d %d %d %d %d %d %d %d %d %d %d %d %d\n",
-		   name, &NbTimeStep,
-		   &NbSP, &NbVP, &NbTP, &NbSL, &NbVL, &NbTL,
-		   &NbST, &NbVT, &NbTT, &NbSQ, &NbVQ, &NbTQ,
-		   &NbSS, &NbVS, &NbTS, &NbSH, &NbVH, &NbTH,
-		   &NbSI, &NbVI, &NbTI, &NbSY, &NbVY, &NbTY,
-		   &NbT2, &t2l, &NbT3, &t3l)){
-	  Msg(GERROR, "Read error");
-	  return false;
-	}
-      }
-      else if(version == 1.4) {
-        Msg(DEBUG, "Detected post-processing view format 1.4");
-        if(!fscanf(fp, "%s %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d "
-		   "%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d "
-		   "%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d\n",
-		   name, &NbTimeStep,
-		   &NbSP, &NbVP, &NbTP, &NbSL, &NbVL, &NbTL,
-		   &NbST, &NbVT, &NbTT, &NbSQ, &NbVQ, &NbTQ,
-		   &NbSS, &NbVS, &NbTS, &NbSH, &NbVH, &NbTH,
-		   &NbSI, &NbVI, &NbTI, &NbSY, &NbVY, &NbTY,
-		   &NbSL2, &NbVL2, &NbTL2, &NbST2, &NbVT2, &NbTT2,
-		   &NbSQ2, &NbVQ2, &NbTQ2, &NbSS2, &NbVS2, &NbTS2,
-		   &NbSH2, &NbVH2, &NbTH2, &NbSI2, &NbVI2, &NbTI2,
-		   &NbSY2, &NbVY2, &NbTY2, &NbT2, &t2l, &NbT3, &t3l)){
-	  Msg(GERROR, "Read error");
-	  return false;
-	}
-      }
-      else {
-        Msg(GERROR, "Unknown post-processing file format (version %g)", version);
-        return false;
-      }
-      
-      for(int i = 0; i < (int)strlen(name); i++)
-        if(name[i] == '^')
-          name[i] = ' ';
-      
-      swap = 0;
-      if(format == LIST_FORMAT_BINARY) {
-	int testone;
-        if(!fread(&testone, sizeof(int), 1, fp)){
-	  Msg(GERROR, "Read error");
-	  return false;
-	}
-        if(testone != 1) {
-          Msg(INFO, "Swapping bytes from binary file");
-          swap = 1;
-        }
-      }
-
-      DataSize = size;
-      
-      // Time values
-      Time = List_CreateFromFile(NbTimeStep, 100, size, fp, format, swap);
-
-      // Note: if nb==0, we still allocates the lists (so that they
-      // are ready to be filled later, e.g. in plugins)
-
-#define LCD List_CreateFromFile(nb, 1000, size, fp, format, swap)
-      int nb;
-      // Points
-      nb = NbSP ? NbSP * (NbTimeStep * 1 + 3) : 0; SP = LCD;
-      nb = NbVP ? NbVP * (NbTimeStep * 3 + 3) : 0; VP = LCD;
-      nb = NbTP ? NbTP * (NbTimeStep * 9 + 3) : 0; TP = LCD;
-
-      // Lines
-      nb = NbSL ? NbSL * (NbTimeStep * 2 * 1 + 6) : 0; SL = LCD;
-      nb = NbVL ? NbVL * (NbTimeStep * 2 * 3 + 6) : 0; VL = LCD;
-      nb = NbTL ? NbTL * (NbTimeStep * 2 * 9 + 6) : 0; TL = LCD;
-
-      // Triangles
-      nb = NbST ? NbST * (NbTimeStep * 3 * 1 + 9) : 0; ST = LCD;
-      nb = NbVT ? NbVT * (NbTimeStep * 3 * 3 + 9) : 0; VT = LCD;
-      nb = NbTT ? NbTT * (NbTimeStep * 3 * 9 + 9) : 0; TT = LCD;
-
-      // Quadrangles
-      nb = NbSQ ? NbSQ * (NbTimeStep * 4 * 1 + 12) : 0; SQ = LCD;
-      nb = NbVQ ? NbVQ * (NbTimeStep * 4 * 3 + 12) : 0; VQ = LCD;
-      nb = NbTQ ? NbTQ * (NbTimeStep * 4 * 9 + 12) : 0; TQ = LCD;
-
-      // Tetrahedra
-      nb = NbSS ? NbSS * (NbTimeStep * 4 * 1 + 12) : 0; SS = LCD;
-      nb = NbVS ? NbVS * (NbTimeStep * 4 * 3 + 12) : 0; VS = LCD;
-      nb = NbTS ? NbTS * (NbTimeStep * 4 * 9 + 12) : 0; TS = LCD;
-
-      // Hexahedra
-      nb = NbSH ? NbSH * (NbTimeStep * 8 * 1 + 24) : 0; SH = LCD;
-      nb = NbVH ? NbVH * (NbTimeStep * 8 * 3 + 24) : 0; VH = LCD;
-      nb = NbTH ? NbTH * (NbTimeStep * 8 * 9 + 24) : 0; TH = LCD;
-
-      // Prisms
-      nb = NbSI ? NbSI * (NbTimeStep * 6 * 1 + 18) : 0; SI = LCD;
-      nb = NbVI ? NbVI * (NbTimeStep * 6 * 3 + 18) : 0; VI = LCD;
-      nb = NbTI ? NbTI * (NbTimeStep * 6 * 9 + 18) : 0; TI = LCD;
-
-      // Pyramids
-      nb = NbSY ? NbSY * (NbTimeStep * 5 * 1 + 15) : 0; SY = LCD;
-      nb = NbVY ? NbVY * (NbTimeStep * 5 * 3 + 15) : 0; VY = LCD;
-      nb = NbTY ? NbTY * (NbTimeStep * 5 * 9 + 15) : 0; TY = LCD;
-
-      // 2nd order Lines
-      nb = NbSL2 ? NbSL2 * (NbTimeStep * 3 * 1 + 9) : 0; SL2 = LCD;
-      nb = NbVL2 ? NbVL2 * (NbTimeStep * 3 * 3 + 9) : 0; VL2 = LCD;
-      nb = NbTL2 ? NbTL2 * (NbTimeStep * 3 * 9 + 9) : 0; TL2 = LCD;
-
-      // 2nd order Triangles
-      nb = NbST2 ? NbST2 * (NbTimeStep * 6 * 1 + 18) : 0; ST2 = LCD;
-      nb = NbVT2 ? NbVT2 * (NbTimeStep * 6 * 3 + 18) : 0; VT2 = LCD;
-      nb = NbTT2 ? NbTT2 * (NbTimeStep * 6 * 9 + 18) : 0; TT2 = LCD;
-
-      // 2nd order Quadrangles
-      nb = NbSQ2 ? NbSQ2 * (NbTimeStep * 9 * 1 + 27) : 0; SQ2 = LCD;
-      nb = NbVQ2 ? NbVQ2 * (NbTimeStep * 9 * 3 + 27) : 0; VQ2 = LCD;
-      nb = NbTQ2 ? NbTQ2 * (NbTimeStep * 9 * 9 + 27) : 0; TQ2 = LCD;
-
-      // 2nd order Tetrahedra
-      nb = NbSS2 ? NbSS2 * (NbTimeStep * 10 * 1 + 30) : 0; SS2 = LCD;
-      nb = NbVS2 ? NbVS2 * (NbTimeStep * 10 * 3 + 30) : 0; VS2 = LCD;
-      nb = NbTS2 ? NbTS2 * (NbTimeStep * 10 * 9 + 30) : 0; TS2 = LCD;
-
-      // 2nd order Hexahedra
-      nb = NbSH2 ? NbSH2 * (NbTimeStep * 27 * 1 + 81) : 0; SH2 = LCD;
-      nb = NbVH2 ? NbVH2 * (NbTimeStep * 27 * 3 + 81) : 0; VH2 = LCD;
-      nb = NbTH2 ? NbTH2 * (NbTimeStep * 27 * 9 + 81) : 0; TH2 = LCD;
-
-      // 2nd order Prisms
-      nb = NbSI2 ? NbSI2 * (NbTimeStep * 18 * 1 + 54) : 0; SI2 = LCD;
-      nb = NbVI2 ? NbVI2 * (NbTimeStep * 18 * 3 + 54) : 0; VI2 = LCD;
-      nb = NbTI2 ? NbTI2 * (NbTimeStep * 18 * 9 + 54) : 0; TI2 = LCD;
-
-      // 2nd order Pyramids
-      nb = NbSY2 ? NbSY2 * (NbTimeStep * 14 * 1 + 42) : 0; SY2 = LCD;
-      nb = NbVY2 ? NbVY2 * (NbTimeStep * 14 * 3 + 42) : 0; VY2 = LCD;
-      nb = NbTY2 ? NbTY2 * (NbTimeStep * 14 * 9 + 42) : 0; TY2 = LCD;
-#undef LCD
-
-      // 2D strings
-      nb = NbT2 ? NbT2 * 4 : 0;
-      T2D = List_CreateFromFile(nb, 100, size, fp, format, swap);
-      if(version <= 1.2)
-	T2C = List_CreateFromFileOld(t2l, 100, sizeof(char), fp, format, swap);
-      else
-	T2C = List_CreateFromFile(t2l, 100, sizeof(char), fp, format, swap);
-
-      // 3D strings
-      nb = NbT3 ? NbT3 * 5 : 0;
-      T3D = List_CreateFromFile(nb, 100, size, fp, format, swap);
-      if(version <= 1.2)
-	T3C = List_CreateFromFileOld(t3l, 100, sizeof(char), fp, format, swap);
-      else
-	T3C = List_CreateFromFile(t3l, 100, sizeof(char), fp, format, swap);
-
-      Msg(DEBUG,
-          "Read View '%s' (%d TimeSteps): "
-	  "SP(%d/%d) VP(%d/%d) TP(%d/%d) "
-	  "SL(%d/%d) VL(%d/%d) TL(%d/%d) "
-	  "ST(%d/%d) VT(%d/%d) TT(%d/%d) "
-	  "SQ(%d/%d) VQ(%d/%d) TQ(%d/%d) "
-	  "SS(%d/%d) VS(%d/%d) TS(%d/%d) "
-	  "SH(%d/%d) VH(%d/%d) TH(%d/%d) "
-	  "SI(%d/%d) VI(%d/%d) TI(%d/%d) "
-	  "SY(%d/%d) VY(%d/%d) TY(%d/%d) " 
-	  "SL2(%d/%d) VL2(%d/%d) TL2(%d/%d) "
-	  "ST2(%d/%d) VT2(%d/%d) TT2(%d/%d) "
-	  "SQ2(%d/%d) VQ2(%d/%d) TQ2(%d/%d) "
-	  "SS2(%d/%d) VS2(%d/%d) TS2(%d/%d) " 
-	  "SH2(%d/%d) VH2(%d/%d) TH2(%d/%d) "
-	  "SI2(%d/%d) VI2(%d/%d) TI2(%d/%d) "
-	  "SY2(%d/%d) VY2(%d/%d) TY2(%d/%d) "
-	  "T2(%d/%d/%d) T3(%d/%d/%d) ", 
-	  name, NbTimeStep,
-          NbSP, List_Nbr(SP), NbVP, List_Nbr(VP), NbTP, List_Nbr(TP),
-          NbSL, List_Nbr(SL), NbVL, List_Nbr(VL), NbTL, List_Nbr(TL),
-          NbST, List_Nbr(ST), NbVT, List_Nbr(VT), NbTT, List_Nbr(TT),
-          NbSQ, List_Nbr(SQ), NbVQ, List_Nbr(VQ), NbTQ, List_Nbr(TQ),
-          NbSS, List_Nbr(SS), NbVS, List_Nbr(VS), NbTS, List_Nbr(TS),
-          NbSH, List_Nbr(SH), NbVH, List_Nbr(VH), NbTH, List_Nbr(TH),
-          NbSI, List_Nbr(SI), NbVI, List_Nbr(VI), NbTI, List_Nbr(TI),
-          NbSY, List_Nbr(SY), NbVY, List_Nbr(VY), NbTY, List_Nbr(TY),
-          NbSL2, List_Nbr(SL2), NbVL2, List_Nbr(VL2), NbTL2, List_Nbr(TL2),
-          NbST2, List_Nbr(ST2), NbVT2, List_Nbr(VT2), NbTT2, List_Nbr(TT2),
-          NbSQ2, List_Nbr(SQ2), NbVQ2, List_Nbr(VQ2), NbTQ2, List_Nbr(TQ2),
-          NbSS2, List_Nbr(SS2), NbVS2, List_Nbr(VS2), NbTS2, List_Nbr(TS2),
-          NbSH2, List_Nbr(SH2), NbVH2, List_Nbr(VH2), NbTH2, List_Nbr(TH2),
-          NbSI2, List_Nbr(SI2), NbVI2, List_Nbr(VI2), NbTI2, List_Nbr(TI2),
-          NbSY2, List_Nbr(SY2), NbVY2, List_Nbr(VY2), NbTY2, List_Nbr(TY2),
-	  NbT2, List_Nbr(T2D), List_Nbr(T2C), 
-	  NbT3, List_Nbr(T3D), List_Nbr(T3C));
-      
-      finalize();
+    if(testone != 1) {
+      Msg(INFO, "Swapping bytes from binary file");
+      swap = 1;
     }
-
-    do {
-      if(!fgets(str, 256, fp) || feof(fp))
-        Msg(GERROR, "Prematured end of file");
-    } while(str[0] != '$');
-    
   }
+  
+  DataSize = size;
+  
+  // Time values
+  Time = List_CreateFromFile(NbTimeStep, 100, size, fp, format, swap);
+  
+  // Note: if nb==0, we still allocates the lists (so that they
+  // are ready to be filled later, e.g. in plugins)
+  
+#define LCD List_CreateFromFile(nb, 1000, size, fp, format, swap)
+  int nb;
+  // Points
+  nb = NbSP ? NbSP * (NbTimeStep * 1 + 3) : 0; SP = LCD;
+  nb = NbVP ? NbVP * (NbTimeStep * 3 + 3) : 0; VP = LCD;
+  nb = NbTP ? NbTP * (NbTimeStep * 9 + 3) : 0; TP = LCD;
+  
+  // Lines
+  nb = NbSL ? NbSL * (NbTimeStep * 2 * 1 + 6) : 0; SL = LCD;
+  nb = NbVL ? NbVL * (NbTimeStep * 2 * 3 + 6) : 0; VL = LCD;
+  nb = NbTL ? NbTL * (NbTimeStep * 2 * 9 + 6) : 0; TL = LCD;
+  
+  // Triangles
+  nb = NbST ? NbST * (NbTimeStep * 3 * 1 + 9) : 0; ST = LCD;
+  nb = NbVT ? NbVT * (NbTimeStep * 3 * 3 + 9) : 0; VT = LCD;
+  nb = NbTT ? NbTT * (NbTimeStep * 3 * 9 + 9) : 0; TT = LCD;
+  
+  // Quadrangles
+  nb = NbSQ ? NbSQ * (NbTimeStep * 4 * 1 + 12) : 0; SQ = LCD;
+  nb = NbVQ ? NbVQ * (NbTimeStep * 4 * 3 + 12) : 0; VQ = LCD;
+  nb = NbTQ ? NbTQ * (NbTimeStep * 4 * 9 + 12) : 0; TQ = LCD;
+  
+  // Tetrahedra
+  nb = NbSS ? NbSS * (NbTimeStep * 4 * 1 + 12) : 0; SS = LCD;
+  nb = NbVS ? NbVS * (NbTimeStep * 4 * 3 + 12) : 0; VS = LCD;
+  nb = NbTS ? NbTS * (NbTimeStep * 4 * 9 + 12) : 0; TS = LCD;
+  
+  // Hexahedra
+  nb = NbSH ? NbSH * (NbTimeStep * 8 * 1 + 24) : 0; SH = LCD;
+  nb = NbVH ? NbVH * (NbTimeStep * 8 * 3 + 24) : 0; VH = LCD;
+  nb = NbTH ? NbTH * (NbTimeStep * 8 * 9 + 24) : 0; TH = LCD;
+  
+  // Prisms
+  nb = NbSI ? NbSI * (NbTimeStep * 6 * 1 + 18) : 0; SI = LCD;
+  nb = NbVI ? NbVI * (NbTimeStep * 6 * 3 + 18) : 0; VI = LCD;
+  nb = NbTI ? NbTI * (NbTimeStep * 6 * 9 + 18) : 0; TI = LCD;
+  
+  // Pyramids
+  nb = NbSY ? NbSY * (NbTimeStep * 5 * 1 + 15) : 0; SY = LCD;
+  nb = NbVY ? NbVY * (NbTimeStep * 5 * 3 + 15) : 0; VY = LCD;
+  nb = NbTY ? NbTY * (NbTimeStep * 5 * 9 + 15) : 0; TY = LCD;
+  
+  // 2nd order Lines
+  nb = NbSL2 ? NbSL2 * (NbTimeStep * 3 * 1 + 9) : 0; SL2 = LCD;
+  nb = NbVL2 ? NbVL2 * (NbTimeStep * 3 * 3 + 9) : 0; VL2 = LCD;
+  nb = NbTL2 ? NbTL2 * (NbTimeStep * 3 * 9 + 9) : 0; TL2 = LCD;
+  
+  // 2nd order Triangles
+  nb = NbST2 ? NbST2 * (NbTimeStep * 6 * 1 + 18) : 0; ST2 = LCD;
+  nb = NbVT2 ? NbVT2 * (NbTimeStep * 6 * 3 + 18) : 0; VT2 = LCD;
+  nb = NbTT2 ? NbTT2 * (NbTimeStep * 6 * 9 + 18) : 0; TT2 = LCD;
+  
+  // 2nd order Quadrangles
+  nb = NbSQ2 ? NbSQ2 * (NbTimeStep * 9 * 1 + 27) : 0; SQ2 = LCD;
+  nb = NbVQ2 ? NbVQ2 * (NbTimeStep * 9 * 3 + 27) : 0; VQ2 = LCD;
+  nb = NbTQ2 ? NbTQ2 * (NbTimeStep * 9 * 9 + 27) : 0; TQ2 = LCD;
+  
+  // 2nd order Tetrahedra
+  nb = NbSS2 ? NbSS2 * (NbTimeStep * 10 * 1 + 30) : 0; SS2 = LCD;
+  nb = NbVS2 ? NbVS2 * (NbTimeStep * 10 * 3 + 30) : 0; VS2 = LCD;
+  nb = NbTS2 ? NbTS2 * (NbTimeStep * 10 * 9 + 30) : 0; TS2 = LCD;
+  
+  // 2nd order Hexahedra
+  nb = NbSH2 ? NbSH2 * (NbTimeStep * 27 * 1 + 81) : 0; SH2 = LCD;
+  nb = NbVH2 ? NbVH2 * (NbTimeStep * 27 * 3 + 81) : 0; VH2 = LCD;
+  nb = NbTH2 ? NbTH2 * (NbTimeStep * 27 * 9 + 81) : 0; TH2 = LCD;
+  
+  // 2nd order Prisms
+  nb = NbSI2 ? NbSI2 * (NbTimeStep * 18 * 1 + 54) : 0; SI2 = LCD;
+  nb = NbVI2 ? NbVI2 * (NbTimeStep * 18 * 3 + 54) : 0; VI2 = LCD;
+  nb = NbTI2 ? NbTI2 * (NbTimeStep * 18 * 9 + 54) : 0; TI2 = LCD;
+  
+  // 2nd order Pyramids
+  nb = NbSY2 ? NbSY2 * (NbTimeStep * 14 * 1 + 42) : 0; SY2 = LCD;
+  nb = NbVY2 ? NbVY2 * (NbTimeStep * 14 * 3 + 42) : 0; VY2 = LCD;
+  nb = NbTY2 ? NbTY2 * (NbTimeStep * 14 * 9 + 42) : 0; TY2 = LCD;
+#undef LCD
+  
+  // 2D strings
+  nb = NbT2 ? NbT2 * 4 : 0;
+  T2D = List_CreateFromFile(nb, 100, size, fp, format, swap);
+  if(version <= 1.2)
+    T2C = List_CreateFromFileOld(t2l, 100, sizeof(char), fp, format, swap);
+  else
+    T2C = List_CreateFromFile(t2l, 100, sizeof(char), fp, format, swap);
+  
+  // 3D strings
+  nb = NbT3 ? NbT3 * 5 : 0;
+  T3D = List_CreateFromFile(nb, 100, size, fp, format, swap);
+  if(version <= 1.2)
+    T3C = List_CreateFromFileOld(t3l, 100, sizeof(char), fp, format, swap);
+  else
+    T3C = List_CreateFromFile(t3l, 100, sizeof(char), fp, format, swap);
+  
+  Msg(DEBUG,
+      "Read View '%s' (%d TimeSteps): "
+      "SP(%d/%d) VP(%d/%d) TP(%d/%d) "
+      "SL(%d/%d) VL(%d/%d) TL(%d/%d) "
+      "ST(%d/%d) VT(%d/%d) TT(%d/%d) "
+      "SQ(%d/%d) VQ(%d/%d) TQ(%d/%d) "
+      "SS(%d/%d) VS(%d/%d) TS(%d/%d) "
+      "SH(%d/%d) VH(%d/%d) TH(%d/%d) "
+      "SI(%d/%d) VI(%d/%d) TI(%d/%d) "
+      "SY(%d/%d) VY(%d/%d) TY(%d/%d) " 
+      "SL2(%d/%d) VL2(%d/%d) TL2(%d/%d) "
+      "ST2(%d/%d) VT2(%d/%d) TT2(%d/%d) "
+      "SQ2(%d/%d) VQ2(%d/%d) TQ2(%d/%d) "
+      "SS2(%d/%d) VS2(%d/%d) TS2(%d/%d) " 
+      "SH2(%d/%d) VH2(%d/%d) TH2(%d/%d) "
+      "SI2(%d/%d) VI2(%d/%d) TI2(%d/%d) "
+      "SY2(%d/%d) VY2(%d/%d) TY2(%d/%d) "
+      "T2(%d/%d/%d) T3(%d/%d/%d) ", 
+      name, NbTimeStep,
+      NbSP, List_Nbr(SP), NbVP, List_Nbr(VP), NbTP, List_Nbr(TP),
+      NbSL, List_Nbr(SL), NbVL, List_Nbr(VL), NbTL, List_Nbr(TL),
+      NbST, List_Nbr(ST), NbVT, List_Nbr(VT), NbTT, List_Nbr(TT),
+      NbSQ, List_Nbr(SQ), NbVQ, List_Nbr(VQ), NbTQ, List_Nbr(TQ),
+      NbSS, List_Nbr(SS), NbVS, List_Nbr(VS), NbTS, List_Nbr(TS),
+      NbSH, List_Nbr(SH), NbVH, List_Nbr(VH), NbTH, List_Nbr(TH),
+      NbSI, List_Nbr(SI), NbVI, List_Nbr(VI), NbTI, List_Nbr(TI),
+      NbSY, List_Nbr(SY), NbVY, List_Nbr(VY), NbTY, List_Nbr(TY),
+      NbSL2, List_Nbr(SL2), NbVL2, List_Nbr(VL2), NbTL2, List_Nbr(TL2),
+      NbST2, List_Nbr(ST2), NbVT2, List_Nbr(VT2), NbTT2, List_Nbr(TT2),
+      NbSQ2, List_Nbr(SQ2), NbVQ2, List_Nbr(VQ2), NbTQ2, List_Nbr(TQ2),
+      NbSS2, List_Nbr(SS2), NbVS2, List_Nbr(VS2), NbTS2, List_Nbr(TS2),
+      NbSH2, List_Nbr(SH2), NbVH2, List_Nbr(VH2), NbTH2, List_Nbr(TH2),
+      NbSI2, List_Nbr(SI2), NbVI2, List_Nbr(VI2), NbTI2, List_Nbr(TI2),
+      NbSY2, List_Nbr(SY2), NbVY2, List_Nbr(VY2), NbTY2, List_Nbr(TY2),
+      NbT2, List_Nbr(T2D), List_Nbr(T2C), 
+      NbT3, List_Nbr(T3D), List_Nbr(T3C));
+  
+  setName(name);
+  finalize();
 
-  fclose(fp);
-  setFileName(filename);
   return true;
 }
 
-
 static void writeTimePOS(FILE *fp, List_T *list)
 {
   if(List_Nbr(list) > 1){
@@ -777,4 +718,3 @@ bool PViewDataList::writeMSH(std::string name)
   fclose(fp);
   return true;
 }
-
diff --git a/Post/PView.h b/Post/PView.h
index b3e8ed825ecdfbf085ccb869378ae039f3573463..4e6523adfe022df57ae2e53fb52f0b52180243e9 100644
--- a/Post/PView.h
+++ b/Post/PView.h
@@ -24,7 +24,6 @@
 #include <string>
 #include "VertexArray.h"
 #include "SmoothData.h"
-#include "AdaptiveViews.h"
 #include "PViewData.h"
 #include "PViewOptions.h"
 
@@ -32,17 +31,17 @@
 class PView{
  private:
   static int _globalNum;
-  // unique tag of the view
+  // unique tag of the view (> 0)
   int _num;
   // index of the view in the current view list
   int _index;
   // flag to mark that the view has changed
   bool _changed;
-  // flag to mark that the view is an alias of another view
+  // tag of the source view if this view is an alias, zero otherwise
   int _aliasOf;
   // reference counter mark that some other views link to this one
   int _links;
-  // eye position
+  // eye position (for transparency sorting)
   SPoint3 _eye;
   // the options
   PViewOptions *_options;
@@ -53,11 +52,8 @@ class PView{
   PView(bool allocate=true);
   // alias constructor
   PView(PView *ref, bool copyOptions=true);
+  // default destructor
   ~PView();
-  // the static list of all loaded views
-  static std::vector<PView*> list;
-  // the current view
-  static PView *current();
   PViewOptions *getOptions(){ return _options; }  
   PViewData *getData(){ return _data; }
   int getNum(){ return _num; }
@@ -65,25 +61,24 @@ class PView{
   void setIndex(int val){ _index = val; }
   bool getChanged(){ return _changed; }
   void setChanged(bool val);
-  int& getLinks(){ return _links; }
+  int &getLinks(){ return _links; }
   int getAliasOf(){ return _aliasOf; }
   SPoint3 &getEye(){ return _eye; }
   void setEye(SPoint3 &p){ _eye = p; }
-  void setGlobalResolutionLevel(int level)
-  {
-    //if(adaptive) adaptive->setGlobalResolutionLevel(this, level);
-  }
-  void setAdaptiveResolutionLevel(int level, GMSH_Post_Plugin *plugin = 0)
-  {
-    //if(adaptive) adaptive->setAdaptiveResolutionLevel(this, level, plugin);
-  }
+
+  // the static list of all loaded views
+  static std::vector<PView*> list;
+  // the current view
+  static PView *current();
+  // read view(s) in list format from a file
+  static bool read(std::string filename, int fileIndex=-1);
+  // combine view
+  static void combine(bool time, int how, bool remove);
 
   // vertex arrays to draw triangles and lines efficiently
   VertexArray *va_points, *va_lines, *va_triangles, *va_vectors;
   // smoothed normals
   smooth_normals *normals;
-  // adaptative rendering for high-order datasets
-  Adaptive_Post_View *adaptive;
 };
 
 #endif
@@ -32,17 +31,17 @@
   int _aliasOf;
-  // reference counter mark that some other views link to this one
+  // reference counter (how many views link to this one)
   int _links;